/*
	FreeRTOS V4.0.1 - Copyright (C) 2003 - 2006 Richard Barry.

	This file is part of the FreeRTOS distribution.

	FreeRTOS is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************
	See http://www.FreeRTOS.org for documentation, latest information, license
	and contact details.  Please ensure to read the configuration and relevant
	port sections of the online documentation.
	***************************************************************************
*/

// TODO - Par & Serial Tests need to be ripped out - maybe go ahead and
// modify ser test for uart....
// note that flash is for flashing leds...

/*
 * Creates all the demo application tasks, then starts the scheduler.  The WEB
 * documentation provides more details of the demo application tasks.
 *
 * This demo is configured to execute on the ES449 prototyping board from
 * SoftBaugh. The ES449 has a built in LCD display and a single built in user
 * LED.  Therefore, in place of flashing an LED, the 'flash' and 'check' tasks
 * toggle '*' characters on the LCD.  The left most '*' represents LED 0, the
 * next LED 1, etc.
 *
 * Main. c also creates a task called 'Check'.  This only executes every three
 * seconds but has the highest priority so is guaranteed to get processor time.
 * Its main function is to check that all the other tasks are still operational.
 * Each task that does not flash an LED maintains a unique count that is
 * incremented each time the task successfully completes its function.  Should
 * any error occur within such a task the count is permanently halted.  The
 * 'check' task inspects the count of each task to ensure it has changed since
 * the last time the check task executed.  If all the count variables have
 * changed all the tasks are still executing error free, and the check task
 * toggles an LED with a three second period.  Should any task contain an error
 * at any time the LED toggle rate will increase to 500ms.
 *
 * Please read the documentation for the MSP430 port available on
 * http://www.FreeRTOS.org.
 */

/* Standard includes. */
#include <stdlib.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Demo application includes. */
#include "partest.h"
#include "flash.h"
#include "integer.h"
#include "comtest2.h"
#include "PollQ.h"
#include "dynamic.h"

/* Constants required for hardware setup. */
#define mainALL_BITS_OUTPUT		( ( unsigned portCHAR ) 0xff )
#define mainMAX_FREQUENCY		( ( unsigned portCHAR ) 121 )

/* Constants that define the LED's used by the various tasks. [in this case
the '*' characters on the LCD represent LED's] */
#define mainCHECK_LED			( 4 )
#define mainCOM_TEST_LED		( 10 )

/* Demo task priorities. */
#define mainCHECK_TASK_PRIORITY			( tskIDLE_PRIORITY + 3 )
#define mainCOM_TEST_PRIORITY			( tskIDLE_PRIORITY + 2 )
#define mainQUEUE_POLL_PRIORITY			( tskIDLE_PRIORITY + 2 )
#define mainLED_TASK_PRIORITY			( tskIDLE_PRIORITY + 1 )

/* Baud rate used by the COM test tasks. */
#define mainCOM_TEST_BAUD_RATE			( ( unsigned portLONG ) 115200L )

/* The frequency at which the 'Check' tasks executes.  See the comments at the
top of the page.  When the system is operating error free the 'Check' task
toggles an LED every three seconds.  If an error is discovered in any task the
rate is increased to 500 milliseconds.  [in this case the '*' characters on the
LCD represent LED's]*/
#define mainNO_ERROR_CHECK_DELAY		( ( portTickType ) 3000 / portTICK_RATE_MS  )
#define mainERROR_CHECK_DELAY			( ( portTickType ) 500 / portTICK_RATE_MS  )

/* The constants used in the calculation. */
#define intgCONST1				( ( portLONG ) 123 )
#define intgCONST2				( ( portLONG ) 234567 )
#define intgCONST3				( ( portLONG ) -3 )
#define intgCONST4				( ( portLONG ) 7 )
#define intgEXPECTED_ANSWER		( ( ( intgCONST1 + intgCONST2 ) * intgCONST3 ) / intgCONST4 )

/*
 * The function that implements the Check task.  See the comments at the head
 * of the page for implementation details.
 */
static void vErrorChecks( void *pvParameters );

/*
 * Called by the Check task.  Returns pdPASS if all the other tasks are found
 * to be operating without error - otherwise returns pdFAIL.
 */
static portSHORT prvCheckOtherTasksAreStillRunning( void );

/*
 * Perform the hardware setup required by the ES449 in order to run the demo
 * application.
 */
static void prvSetupHardware( void );


portBASE_TYPE xLocalError = pdFALSE;

/*-----------------------------------------------------------*/

/* 
 * Start the demo application tasks - then start the real time scheduler.
 */
int main( void )
{
	/* Setup the hardware ready for the demo. */
	prvSetupHardware();
	vParTestInitialise();		// '169 

	/* Start the standard demo application tasks. */
	vStartLEDFlashTasks( mainLED_TASK_PRIORITY );	// '169 - 4 of 8 tasks
	vStartMathTasks( tskIDLE_PRIORITY );
	vStartIntegerMathTasks( tskIDLE_PRIORITY );
	vAltStartComTestTasks( mainCOM_TEST_PRIORITY, mainCOM_TEST_BAUD_RATE, mainCOM_TEST_LED - 1 );  // '169 - 2 of 8 tasks
	vStartPolledQueueTasks( mainQUEUE_POLL_PRIORITY );
	vStartBlockingQueueTasks( tskIDLE_PRIORITY );
	vStartDynamicPriorityTasks();

	/* Start the 'Check' task which is defined in this file. */
	// '169 - 1 of 8 tasks
	xTaskCreate( vErrorChecks, ( const signed portCHAR * const ) "Check", configMINIMAL_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY, NULL );	

	/* Start the scheduler. */
	// '169 - IDLE task is '8th' task so to speak
	vTaskStartScheduler();

	/* As the scheduler has been started the demo applications tasks will be
	executing and we should never get here! */
	return 0;
}
/*-----------------------------------------------------------*/

static portTASK_FUNCTION( vErrorChecks, pvParameters )
{
portTickType xDelayPeriod = mainNO_ERROR_CHECK_DELAY;

	/* Cycle for ever, delaying then checking all the other tasks are still
	operating without error. */
	for( ;; )
	{
		/* Wait until it is time to check again.  The time we wait here depends
		on whether an error has been detected or not.  When an error is
		detected the time is shortened resulting in a faster LED flash rate. */
		vTaskDelay( xDelayPeriod );

		/* See if the other tasks are all ok. */
		if( prvCheckOtherTasksAreStillRunning() != pdPASS )
		{
			/* An error occurred in one of the tasks so shorten the delay
			period - which has the effect of increasing the frequency of the
			LED toggle. */
			xDelayPeriod = mainERROR_CHECK_DELAY;
		}

		/* Flash! */
//		vParTestToggleLED( mainCHECK_LED );
	}
}
/*-----------------------------------------------------------*/

static portSHORT prvCheckOtherTasksAreStillRunning( void )
{
static portSHORT sNoErrorFound = pdTRUE;

	/* The demo tasks maintain a count that increments every cycle of the task
	provided that the task has never encountered an error.  This function
	checks the counts maintained by the tasks to ensure they are still being
	incremented.  A count remaining at the same value between calls therefore
	indicates that an error has been detected.  Only tasks that do not flash
	an LED are checked. */

	if( xAreIntegerMathsTaskStillRunning() != pdTRUE )
	{
		sNoErrorFound = pdFALSE;
	}

	if( xAreComTestTasksStillRunning() != pdTRUE )
	{
		sNoErrorFound = pdFALSE;
	}

	if( xArePollingQueuesStillRunning() != pdTRUE )
	{
		sNoErrorFound = pdFALSE;
	}

	if( xLocalError == pdTRUE )
	{
		sNoErrorFound = pdFALSE;
	}
	
	return sNoErrorFound;
}
/*-----------------------------------------------------------*/

static void prvSetupHardware( void )
{

// XXX - Not using DCO - replace this with correct clock stuff...

        /* Stop the watchdog. */
	WDTCTL = WDTPW + WDTHOLD;
//	CLK_init();

	BCSCTL1 = DIVA_0 | RSEL2;

// per UG:
	
    do
    {
	unsigned int i;

	IFG1 &= ~OFIFG;
	for (i = 0; i < 0xff; i++)
	{
	    continue;
	}
    }
    while ((IFG1 & OFIFG) != 0);

    // set 8MHz Clk:  (NOTE: this will be different for other
    // frequencies.  If using the DCO - comment this out and look at
    // the commented code below.

    BCSCTL2 = SELM1 | DIVM_0 | SELS | DIVS_3;

    //	IO_init();

    // set I/O as required by your board.  Do this with CAUTION!
    // the following is supplied as an example only.

    // P1:  
    
    P1SEL_bit.P1SEL_0 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_1 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_2 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_3 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_4 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_5 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_6 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_7 = 0;	// 0 is digital I/O

    P1DIR_bit.P1DIR_0 = 0;	// 0 is input
    P1DIR_bit.P1DIR_1 = 0;	// 0 is input
    P1DIR_bit.P1DIR_2 = 0;	// 0 is input
    P1DIR_bit.P1DIR_3 = 0;	// 0 is input
    P1DIR_bit.P1DIR_4 = 0;	// 0 is input
    P1DIR_bit.P1DIR_5 = 0;	// 0 is input
    P1DIR_bit.P1DIR_6 = 0;	// 0 is input
    P1DIR_bit.P1DIR_7 = 0;	// 0 is input

    // P2:  
    
    P2SEL_bit.P2SEL_0 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_1 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_2 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_3 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_4 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_5 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_6 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_7 = 0;	// 0 is digital I/O

    P2DIR_bit.P2DIR_0 = 0;	// 0 is input
    P2DIR_bit.P2DIR_1 = 0;	// 0 is input
    P2DIR_bit.P2DIR_2 = 0;	// 0 is input
    P2DIR_bit.P2DIR_3 = 0;	// 0 is input
    P2DIR_bit.P2DIR_4 = 1;	// 1 is output  -- NC
    P2DIR_bit.P2DIR_5 = 0;	// 0 is input
    P2DIR_bit.P2DIR_6 = 1;	// 1 is output  -- NC
    P2DIR_bit.P2DIR_7 = 1;	// 1 is output  -- NC

    // P3:

    P3SEL_bit.P3SEL_0 = 0;	// 0 is digital I/O
    P3SEL_bit.P3SEL_1 = 0;	// 0 is digital I/O
    P3SEL_bit.P3SEL_2 = 0;	// 0 is digital I/O
    P3SEL_bit.P3SEL_3 = 0;	// 0 is digital I/O
	
    P3DIR_bit.P3DIR_0 = 0;	// 1 is input
    P3DIR_bit.P3DIR_1 = 0;	// 1 is input
    P3DIR_bit.P3DIR_2 = 0;	// 1 is input
    P3DIR_bit.P3DIR_3 = 0;	// 1 is input

    // UART PINS
    
    // P4:  LED's are on P4:
    
    // LED4:  P4.0
    // LED3:  P4.1
    // LED2:  P4.2
    // LED1:  P4.3

    P4SEL_bit.P4SEL_0 = 0;	// 0 is digital I/O
    P4SEL_bit.P4SEL_1 = 0;	// 0 is digital I/O
    P4SEL_bit.P4SEL_2 = 0;	// 0 is digital I/O
    P4SEL_bit.P4SEL_3 = 0;	// 0 is digital I/O
    P4SEL_bit.P4SEL_4 = 0;	// 0 is digital I/O
    P4SEL_bit.P4SEL_5 = 0;	// 0 is digital I/O
    P4SEL_bit.P4SEL_6 = 0;	// 0 is digital I/O
    P4SEL_bit.P4SEL_7 = 0;	// 0 is digital I/O

    P4DIR_bit.P4DIR_0 = 1;	// 1 is output
    P4DIR_bit.P4DIR_1 = 1;	// 1 is output
    P4DIR_bit.P4DIR_2 = 1;	// 1 is output
    P4DIR_bit.P4DIR_3 = 1;	// 1 is output
    P4DIR_bit.P4DIR_4 = 1;	// 1 is output  -- NC
    P4DIR_bit.P4DIR_5 = 0;	// 0 is input
    P4DIR_bit.P4DIR_6 = 0;	// 0 is input
    P4DIR_bit.P4DIR_7 = 1;	// 1 is output  -- NC

    // P5:

    P5SEL_bit.P5SEL_0 = 0;	// 0 is digital I/O
    P5SEL_bit.P5SEL_1 = 0;	// 0 is digital I/O
    P5SEL_bit.P5SEL_2 = 0;	// 0 is digital I/O
    P5SEL_bit.P5SEL_3 = 0;	// 0 is digital I/O
    P5SEL_bit.P5SEL_4 = 0;	// 0 is digital I/O
    P5SEL_bit.P5SEL_5 = 0;	// 0 is digital I/O
    P5SEL_bit.P5SEL_6 = 0;	// 0 is digital I/O
    P5SEL_bit.P5SEL_7 = 0;	// 0 is digital I/O

    P5DIR_bit.P5DIR_0 = 0;	// 0 is input
    P5DIR_bit.P5DIR_1 = 0;	// 0 is input
    P5DIR_bit.P5DIR_2 = 0;	// 0 is input
    P5DIR_bit.P5DIR_3 = 0;	// 0 is input
    P5DIR_bit.P5DIR_4 = 0;	// 0 is input  
    P5DIR_bit.P5DIR_5 = 1;	// 1 is output  -- NC
    P5DIR_bit.P5DIR_6 = 1;	// 1 is output  -- NC
    P5DIR_bit.P5DIR_7 = 1;	// 1 is output  -- NC

    // P6:

    P6SEL_bit.P6SEL_0 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_1 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_2 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_3 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_4 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_5 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_6 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_7 = 0;	// 0 is digital I/O

    P6DIR_bit.P6DIR_0 = 1;	// 1 is output -- NC
    P6DIR_bit.P6DIR_1 = 1;	// 1 is output -- NC
    
    P6DIR_bit.P6DIR_2 = 0;	// 0 is input
    P6DIR_bit.P6DIR_3 = 0;	// 0 is input
    P6DIR_bit.P6DIR_4 = 0;	// 0 is input
    P6DIR_bit.P6DIR_5 = 0;	// 0 is input
    P6DIR_bit.P6DIR_6 = 0;	// 0 is input
    P6DIR_bit.P6DIR_7 = 0;	// 0 is input
  

    // P1:
/*
    P1SEL_bit.P1SEL_5 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_6 = 0;	// 0 is digital I/O
    P1SEL_bit.P1SEL_7 = 0;	// 0 is digital I/O

    P1DIR_bit.P1DIR_5 = 1;	// 1 is output
    P1DIR_bit.P1DIR_6 = 1;	// 1 is output
    P1DIR_bit.P1DIR_7 = 1;	// 1 is output

    // P2:

    P2SEL_bit.P2SEL_0 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_1 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_2 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_3 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_4 = 0;	// 0 is digital I/O

    P2SEL_bit.P2SEL_6 = 0;	// 0 is digital I/O
    P2SEL_bit.P2SEL_7 = 0;	// 0 is digital I/O

    P2DIR_bit.P2DIR_0 = 1;	// 1 is output
    P2DIR_bit.P2DIR_1 = 1;	// 1 is output
    P2DIR_bit.P2DIR_2 = 1;	// 1 is output
    P2DIR_bit.P2DIR_3 = 1;	// 1 is output
    P2DIR_bit.P2DIR_4 = 1;	// 1 is output

    P2DIR_bit.P2DIR_6 = 1;	// 1 is output
    P2DIR_bit.P2DIR_7 = 1;	// 1 is output

    // P3:

    P3SEL_bit.P3SEL_0 = 0;	// 0 is digital I/O
    P3SEL_bit.P3SEL_1 = 0;	// 0 is digital I/O
    P3SEL_bit.P3SEL_2 = 0;	// 0 is digital I/O
    P3SEL_bit.P3SEL_3 = 0;	// 0 is digital I/O
	
    P3DIR_bit.P3DIR_0 = 1;	// 1 is output
    P3DIR_bit.P3DIR_1 = 1;	// 1 is output
    P3DIR_bit.P3DIR_2 = 1;	// 1 is output
    P3DIR_bit.P3DIR_3 = 1;	// 1 is output

    // P4:

    P4SEL_bit.P4SEL_7 = 0;	// 0 is digital I/O
    P4DIR_bit.P4DIR_7 = 1;	// 1 is output

    // P5:

    P5SEL_bit.P5SEL_7 = 0;	// 0 is digital I/O
    P5DIR_bit.P5DIR_7 = 1;	// 1 is output

    // P6:

    P6SEL_bit.P6SEL_0 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_1 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_2 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_3 = 0;	// 0 is digital I/O

    P6SEL_bit.P6SEL_5 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_6 = 0;	// 0 is digital I/O
    P6SEL_bit.P6SEL_7 = 0;	// 0 is digital I/O

    P6DIR_bit.P6DIR_0 = 1;	// 1 is output
    P6DIR_bit.P6DIR_1 = 1;	// 1 is output
    P6DIR_bit.P6DIR_2 = 1;	// 1 is output
    P6DIR_bit.P6DIR_3 = 1;	// 1 is output

    P6DIR_bit.P6DIR_5 = 1;	// 1 is output
    P6DIR_bit.P6DIR_6 = 1;	// 1 is output
    P6DIR_bit.P6DIR_7 = 1;	// 1 is output
*/

	/* Setup DCO+ for ( xtal * D * (N + 1) ) operation. */
//	FLL_CTL0 |= DCOPLUS + XCAP18PF;

	/* X2 DCO frequency, 8MHz nominal DCO */
// XXX - OK - this looks like our master clock...
//	SCFI0 |= FN_4;

	/* (121+1) x 32768 x 2 = 7.99 Mhz */
//	SCFQCTL = mainMAX_FREQUENCY;

	/* Setup the IO.  This is just copied from the demo supplied by SoftBaugh
	for the ES449 demo board. */

//	P1SEL = 0x32;
//	P2SEL = 0x00;
//	P3SEL = 0x00;
//	P4SEL = 0xFC;
//	P5SEL = 0xFF;


// (re)enable the watchdog if you wish

	
}
/*-----------------------------------------------------------*/

/* The idle hook is just a copy of the standard integer maths tasks.  See
Demo/Common/integer.c for rationale. */

// void vApplicationIdleHook( void ) __toplevel
void vApplicationIdleHook( void )
{
/* These variables are all effectively set to constants so they are volatile to
ensure the compiler does not just get rid of them. */
volatile portLONG lValue;
volatile signed portBASE_TYPE *pxTaskHasExecuted;

	/* Keep performing a calculation and checking the result against a constant. */
	for( ;; )
	{
		/* Perform the calculation.  This will store partial value in
		registers, resulting in a good test of the context switch mechanism. */
		lValue = intgCONST1;
		lValue += intgCONST2;

		/* Yield in case cooperative scheduling is being used. */
		#if configUSE_PREEMPTION == 0
		{
			taskYIELD();
		}
		#endif

		/* Finish off the calculation. */
		lValue *= intgCONST3;
		lValue /= intgCONST4;

		/* If the calculation is found to be incorrect we stop setting the
		TaskHasExecuted variable so the check task can see an error has
		occurred. */
		if( lValue != intgEXPECTED_ANSWER ) /*lint !e774 volatile used to prevent this being optimised out. */
		{
			/* Don't bother with mutual exclusion - it is only read from the
			check task and never written. */
			xLocalError = pdTRUE;
		}
		/* Yield in case cooperative scheduling is being used. */
		#if configUSE_PREEMPTION == 0
		{
			taskYIELD();
		}
		#endif
	}
}





